/*******************************************************************************
 * Copyright (C) 2006 Ping Identity Corporation All rights reserved.
 * 
 * This software is licensed under the Open Software License v2.1 (OSL v2.1).
 * 
 * A copy of this license has been provided with the distribution of this
 * software. Additionally, a copy of this license is available at:
 * http://opensource.org/licenses/osl-2.1.php
 *  
 ******************************************************************************/
using System;
using System.Collections;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using Com.PingIdentity.PingFederate.SampleApp.SampleAppUtility;

namespace Com.PingIdentity.PingFederate.SampleApp.IdpSample.web {

	/// <summary>
	/// Handles user login.
	/// </summary>
	public class LoginForm : Page {

		protected HtmlInputText   Password;
		protected HtmlInputButton Submit;
		protected Label           ErrorLabel;
		protected HtmlInputHidden ResumePath;
		
		private String loginType;
		protected System.Web.UI.WebControls.DropDownList userName;
		protected System.Web.UI.HtmlControls.HtmlGenericControl CENTER1;
		private Links samlLinks;

		private void Page_Load(object sender, EventArgs e) {
			//get loginType parameter: global or local
			loginType = Request.QueryString.Get(Constants.LOGIN_TYPE);
			if(loginType == null) 
			{
				loginType = Constants.LOCAL_TYPE;
			}

			//get properties from Idp config file
			Hashtable idpConfigTable;
			if(Session[Constants.IDP_CONFIG_TABLE] == null) 
			{
				idpConfigTable = GetIdpConfigProps(this.Context);
				Session[Constants.IDP_CONFIG_TABLE] = idpConfigTable;
			}
			else 
			{
				idpConfigTable = (Hashtable)Session[Constants.IDP_CONFIG_TABLE];
			}
			HttpContext.Current.Trace.Warn("idpConfigTable has been loaded");

			samlLinks = new Links(idpConfigTable, Request);

			//get resumePath and save it in hidden field
			String resumePath = Request.QueryString.Get(Constants.RESUME_PATH);
			if(resumePath != null) 
			{
				this.ResumePath.Value = resumePath;
			}

			if(!IsPostBack) 
			{	
				PopulateUserNameList(this.Context);
				if(Request["error"] != null)
				{
					ErrorLabel.Text = Request["error"];
				}
			}
			else
			{
				Authenticate();
			}
		}
		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e) {
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() {    
			this.userName.SelectedIndexChanged += new System.EventHandler(this.userName_SelectedIndexChanged);
			this.Submit.ServerClick += new System.EventHandler(this.Submit_ServerClick);
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion

		private void Submit_ServerClick(object sender, EventArgs e) {

			Authenticate();
		}

		/// <summary>
		/// Reads general configuration properties XML file for the
		/// IDP Sample Application and converts it to Hastable object
		/// </summary>
		/// <param name="context">current HttpContext</param>
		/// <returns>Hashtable of user general configuration properties</returns>
		private Hashtable GetIdpConfigProps(HttpContext context) {

			//read properties from Idp config XML file
			String idpConfig  = ConfigurationSettings.AppSettings["config-file"];
			String configPath = context.Server.MapPath("config/" + idpConfig);

			XmlUtility xmlUtility = new XmlUtility();
			Hashtable idpConfigTable = xmlUtility.GetConfigProps(configPath);
			if(idpConfigTable == null) {
				String errorMessage = "Error in " + idpConfig;
				context.Response.Redirect(Constants.WEB_PREFIX + Constants.ERROR_PAGE + errorMessage);
			}
			return idpConfigTable;
		}

		private void PopulateUserNameList(HttpContext context) 	{
			//read properties from Idp users XML file
			String idpUsers  = ConfigurationSettings.AppSettings["users-props-file"];
			String usersPath = context.Server.MapPath("config/" + idpUsers);

			XmlUtility xmlUtility = new XmlUtility();
			ArrayList usersList = xmlUtility.GetPropsFromXmlFile(usersPath);
			if(usersList == null || usersList.Count == 0) 
			{
				String errorMessage = "Error in " + usersList;
				context.Response.Redirect(Constants.WEB_PREFIX + Constants.ERROR_PAGE + errorMessage);
			}
			else
			{
				for(int i = 0; i < usersList.Count; i++) 
				{
					Hashtable user  = (Hashtable)usersList[i];
					String userId = (String)user[Constants.USER_ID];
					userName.Items.Add(userId);
					
				}
			}
		}


		private void Authenticate() 
		{
			String idpUsers = ConfigurationSettings.AppSettings["users-props-file"];
			String usersPath = Server.MapPath("config/" + idpUsers);

			XmlUtility xmlUtility = new XmlUtility();
			Hashtable userInfo = xmlUtility.AuthenticateUser(userName.SelectedValue,
				Password.Value, usersPath);
			if(userInfo != null) 
			{
				userInfo.Remove(Constants.USER_PASSWORD);
				Session[Constants.USER_INFO] = userInfo;
				HttpCookie authCookie = FormsAuthentication.GetAuthCookie(
					userName.SelectedValue, false);
				Response.SetCookie(authCookie);
				if(loginType.Equals(Constants.LOCAL_TYPE)) 
				{
					Response.Redirect(samlLinks.getIdpHomeUrl()); //redirect to MainHandler
					return;
				}
				else if(loginType.Equals(Constants.GLOBAL_TYPE)) 
				{
					String urlToSSOIdpHandler = samlLinks.getSSOIdpHandlerUrl();
					String resumePath = this.ResumePath.Value;
					if(resumePath != null) 
					{
						urlToSSOIdpHandler = samlLinks.addParameter(urlToSSOIdpHandler,
							Constants.RESUME_PATH, resumePath);
					}
					Response.Redirect(urlToSSOIdpHandler);        //redirect to SSOIdpHandler
				}
				else 
				{
					String errorPage = samlLinks.getIdpErrorPageUrl();
					String errorMessage = "Invalid loginType parameter!";
					Response.Redirect(errorPage + errorMessage);
				}
			}
			else 
			{
				ErrorLabel.Text = "Invalid Credentials: Please try again";
			}
		}

		private void userName_SelectedIndexChanged(object sender, System.EventArgs e)
		{
		
		}
	}
}