/*******************************************************************************
 * Copyright (C) 2006 Ping Identity Corporation All rights reserved.
 * 
 * This software is licensed under the Open Software License v2.1 (OSL v2.1).
 * 
 * A copy of this license has been provided with the distribution of this
 * software. Additionally, a copy of this license is available at:
 * http://opensource.org/licenses/osl-2.1.php
 *  
 ******************************************************************************/
using System;
using System.Collections;
using System.Configuration;
using System.IO;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using Com.PingIdentity.PingFederate.SampleApp.SampleAppUtility;
using Com.PingIdentity.PingFederate.SampleApp.SpSample.cs.main;

namespace Com.PingIdentity.PingFederate.SampleApp.SpSample.web {
	/// <summary>
	/// Handles user login.
	/// </summary>
	public class LoginForm : Page {

		protected DropDownList    userName;
		protected HtmlInputText   Password;
		protected HtmlInputButton Submit;
		protected Label           ErrorLabel;
		protected HtmlInputHidden ResumePath;

		private String loginType;
		protected DropDownList idps;
		private Links  samlLinks;
	
		private void Page_Load(object sender, EventArgs e) {
			
			//get loginType parameter: global or local
			loginType = Request.QueryString.Get(Constants.LOGIN_TYPE);
			if(loginType == null) {
				loginType = Constants.LOCAL_TYPE;
			}

			//get properties from Sp config file
			Hashtable spConfigTable;
			if(Session[Constants.SP_CONFIG_TABLE] == null) {
				spConfigTable = GetSpConfigProps(this.Context);
				Session[Constants.SP_CONFIG_TABLE] = spConfigTable;
			}
			else {
				spConfigTable = (Hashtable)Session[Constants.SP_CONFIG_TABLE];
			}

			samlLinks = new Links(spConfigTable, Request);

			//get resumePath and save it in hidden field
			String resumePath = Request.QueryString.Get(Constants.RESUME_PATH);
			if(resumePath != null) {
				this.ResumePath.Value = resumePath;
			}

			if(!IsPostBack) 
			{	
				PopulateUserNameList(this.Context);
				PopulateIdps(this.Context);
				if(Request["error"] != null)
				{
					ErrorLabel.Text = Request["error"];
				}
			}
			else
			{
				Authenticate();
			}
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.idps.SelectedIndexChanged += new System.EventHandler(this.idps_SelectedIndexChanged);
			this.Submit.ServerClick += new System.EventHandler(this.Submit_ServerClick);
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion

		private void Submit_ServerClick(object sender, EventArgs e) {
			Authenticate();
		}

		/// <summary>
		/// Reads general configuration properties XML file for the
		/// SP Sample Application and converts it to Hastable object
		/// </summary>
		/// <param name="context">current HttpContext</param>
		/// <returns>Hashtable of user general configuration properties</returns>
		private Hashtable GetSpConfigProps(HttpContext context) {

			//read properties from Sp config XML file
			String spConfig  = ConfigurationSettings.AppSettings["config-file"];
			String configPath = context.Server.MapPath("config/" + spConfig);

			XmlUtility xmlUtility = new XmlUtility();
			Hashtable spConfigTable = xmlUtility.GetConfigProps(configPath);
			if(spConfigTable == null) {
				String errorMessage = "Error in " + spConfig;
				context.Response.Redirect(Constants.WEB_PREFIX + Constants.ERROR_PAGE + errorMessage);
			}
			return spConfigTable;
		}

		private void PopulateUserNameList(HttpContext context) 	
		{
			//read properties from Idp users XML file
			String idpUsers  = ConfigurationSettings.AppSettings["users-props-file"];
			String usersPath = context.Server.MapPath("config/" + idpUsers);

			XmlUtility xmlUtility = new XmlUtility();
			ArrayList usersList = xmlUtility.GetPropsFromXmlFile(usersPath);
			if(usersList == null || usersList.Count == 0) 
			{
				String errorMessage = "Error in " + usersList;
				context.Response.Redirect(Constants.WEB_PREFIX + Constants.ERROR_PAGE + errorMessage);
			}
			else
			{
				for(int i = 0; i < usersList.Count; i++) 
				{
					Hashtable user  = (Hashtable)usersList[i];
					String userId = (String)user[Constants.USER_ID];
					userName.Items.Add(userId);
					
				}
			}
		}

		private void PopulateIdps(HttpContext context)
		{
			//read properties from Idp connections XML file
			String spConnections  = ConfigurationSettings.AppSettings["connections-file"];
			String connectionsPath = Context.Server.MapPath("config/" + spConnections);
			if (!File.Exists(connectionsPath)) 
			{
				SSOEntity[] idpList = null;
				try 
				{
					SSODirectoryServiceService directoryService = new SSODirectoryServiceService();
					idpList = directoryService.getIDPList();
				}
				catch(Exception e) 
				{
					// Log exception
					HttpContext.Current.Trace.Warn(e.ToString());
					// Display alert
					Response.Write("<script language=\"javascript\">");
					Response.Write("alert('No IdP connections were found, please make sure PingFederate is running and there are connections configured.')");
					Response.Write("</script>");
					return;
				}
				IEnumerator list = idpList.GetEnumerator();
				list.Reset();
				int i=0;
				while(list.MoveNext()) 
				{
					i++;
					SSOEntity idp = (SSOEntity)list.Current;
					String connectionName = (String)idp.entityId;
					String spSSOUrl = null;
					
					spSSOUrl = samlLinks.getSpStartSSOUrl(connectionName,null,null);

					ListItem item = new ListItem(connectionName,spSSOUrl);
					idps.Items.Add(item);
				}
				// Check to see if any connections were obtained
				if(i ==0 ) 
				{
					Response.Write("<script language=\"javascript\">");
					Response.Write("alert('No IdP connections were found, please make sure PingFederate is running and there are connections configured.')");
					Response.Write("</script>");
				}
			}
			else 
			{
				//get properties from Sp connections file
				ArrayList spConnectionsList;
				if(Session[Constants.SP_CONNECT_LIST] == null) 
				{
					spConnectionsList = MainHandler.GetSpConnectionsProps(Context);
					Session[Constants.SP_CONNECT_LIST] = spConnectionsList;
				}
				else 
				{
					spConnectionsList = (ArrayList)Session[Constants.SP_CONNECT_LIST];
				}
				for(int i = 0; i < spConnectionsList.Count; i++) 
				{
					Hashtable connection  = (Hashtable)spConnectionsList[i];
					String connectionName = (String)connection[Constants.CONNECTION_NAME];
					String partnerIdpId   = (String)connection[Constants.CONNECTION_VALUE];
					String SpAdapterId    = (String)connection[Constants.SP_ADAPTER_ID];
					String spSSOUrl       = samlLinks.getSpStartSSOUrl(partnerIdpId, SpAdapterId, null, null);
					ListItem item = new ListItem(connectionName,spSSOUrl);
					idps.Items.Add(item);
				}
			}

		}

		private void Authenticate()
		{
			String spUsers   = ConfigurationSettings.AppSettings["users-props-file"];
			String usersPath = Server.MapPath("config/" + spUsers);

			XmlUtility xmlUtility = new XmlUtility();
			Hashtable userInfo = xmlUtility.AuthenticateUser(userName.SelectedValue,
				Password.Value, usersPath);
			if(userInfo != null) 
			{
				userInfo.Remove(Constants.USER_PASSWORD);
				Session[Constants.USER_INFO] = userInfo;
				HttpCookie authCookie = FormsAuthentication.GetAuthCookie(
					userName.SelectedValue, false);
				Response.SetCookie(authCookie);
				if(loginType.Equals(Constants.LOCAL_TYPE)) 
				{
					// Redirect to SP application protected page
					String mainpage = samlLinks.getSpMainPageUrl();
					Response.Redirect(mainpage);
				}
				else if(loginType.Equals(Constants.GLOBAL_TYPE)) 
				{
					String urlToSSOSpHandler = samlLinks.getSSOSpHandlerUrl();
					String resumePath = this.ResumePath.Value;
					if(resumePath != null) 
					{
						urlToSSOSpHandler = samlLinks.addParameter(urlToSSOSpHandler,
							Constants.RESUME_PATH, resumePath);
					}
					Response.Redirect(urlToSSOSpHandler);  //redirect to SSOSpHandler
				}
				else 
				{
					String errorPage = samlLinks.getSpErrorPageUrl();
					String errorMessage = "Invalid loginType parameter!";
					Response.Redirect(errorPage + errorMessage);
				}
			}
			else 
			{
				ErrorLabel.Text = "Invalid Credentials: Please try again";
			}
		}

		private void idps_SelectedIndexChanged(object sender, System.EventArgs e)
		{
		
		}
	}
}