/*******************************************************************************
 * Copyright (C) 2006 Ping Identity Corporation All rights reserved.
 * 
 * This software is licensed under the Open Software License v2.1 (OSL v2.1).
 * 
 * A copy of this license has been provided with the distribution of this
 * software. Additionally, a copy of this license is available at:
 * http://opensource.org/licenses/osl-2.1.php
 *  
 ******************************************************************************/
using System;
using System.Collections;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.SessionState;
using System.IO;
using Com.PingIdentity.PingFederate.SampleApp.SampleAppUtility;

namespace Com.PingIdentity.PingFederate.SampleApp.SpSample.cs.main {

	/// <summary>
	/// This is the main servlet of the SP Sample Application and handles local login
	/// </summary>
	public class MainHandler : IHttpHandler, IRequiresSessionState 
	{

		public void ProcessRequest(HttpContext context) 
		{

			HttpRequest request		 = context.Request;
			HttpResponse response    = context.Response;
			HttpSessionState session = context.Session;

			//get properties from Sp config file
			Hashtable spConfigTable;
			if(session[Constants.SP_CONFIG_TABLE] == null) 
			{
				spConfigTable = GetSpConfigProps(context);
				session[Constants.SP_CONFIG_TABLE] = spConfigTable;
			}
			else 
			{
				spConfigTable = (Hashtable)session[Constants.SP_CONFIG_TABLE];
			}
			context.Trace.Warn("spConfigTable has been loaded");

			//get properties from Sp connections file
			ArrayList spConnectionsList;
			if(session[Constants.SP_CONNECT_LIST] == null) 
			{
				spConnectionsList = GetSpConnectionsProps(context);
				session[Constants.SP_CONNECT_LIST] = spConnectionsList;
			}
			else 
			{
				spConnectionsList = (ArrayList)session[Constants.SP_CONNECT_LIST];
			}
			context.Trace.Warn("spConnectionsList has been loaded");

			Links samlLinks = samlLinks = new Links(spConfigTable, request);

			//create URL to Login Page
			String loginPage = samlLinks.getSpLoginPageUrl(Constants.LOCAL_TYPE);

			//Get user attributes from session
			Hashtable userInfo = (Hashtable)session[Constants.USER_INFO];
			if((userInfo == null) || (userInfo.Count <= 0)) 
			{
				context.Trace.Warn("userInfo = null, redirect to loginPage");
				response.Redirect(loginPage);
				return;
			}

			//check the logout case
			String cmd = request.QueryString.Get(Constants.PARAM_CMD);
			if( cmd != null && cmd.Equals(Constants.CMD_LOGOUT)) 
			{
				String resumePath = request.QueryString.Get(Constants.RESUME_PATH);
				FormsAuthentication.SignOut();
				session.Abandon();
				context.Trace.Warn("logout case: cmd = logout");
				if(resumePath != null) 
				{
					resumePath = samlLinks.getResumePathUrl(resumePath); //create full path
					response.Redirect(resumePath);
				}
				else 
				{
					response.Redirect(loginPage);
				}
				return;
			}

			// Redirect to SP application protected page
			String mainpage = samlLinks.getSpMainPageUrl();
			response.Redirect(mainpage);
		}

		/// <summary>
		/// Reads general configuration properties XML file for the
		/// SP Sample Application and converts it to Hastable object
		/// </summary>
		/// <param name="context">current HttpContext</param>
		/// <returns>Hashtable of user general configuration properties</returns>
		private Hashtable GetSpConfigProps(HttpContext context) 
		{

			//read properties from Sp config XML file
			String spConfig  = ConfigurationSettings.AppSettings["config-file"];
			String configPath = context.Server.MapPath("config/" + spConfig);

			XmlUtility xmlUtility = new XmlUtility();
			Hashtable spConfigTable = xmlUtility.GetConfigProps(configPath);
			if(spConfigTable == null) 
			{
				String errorMessage = "Error in " + spConfig;
				context.Response.Redirect(Constants.WEB_PREFIX + Constants.ERROR_PAGE + errorMessage);
			}
			return spConfigTable;
		}

		/// <summary>
		/// Reads connection properties XML file for specifying Idp Connections
		/// and converts it to ArrayList of Hashtables with user Idp connections
		/// </summary>
		/// <param name="context">current HttpContext</param>
		/// <returns>ArrayList of user connection properties</returns>
		public static ArrayList GetSpConnectionsProps(HttpContext context) 
		{

			//read properties from Sp connections XML file
			String spConnections  = ConfigurationSettings.AppSettings["connections-file"];
			String connectionsPath = context.Server.MapPath("config/" + spConnections);

			if (File.Exists(connectionsPath)) 
			{
				XmlUtility xmlUtility = new XmlUtility();
				ArrayList connectionsList = xmlUtility.GetPropsFromXmlFile(connectionsPath);
				if(connectionsList == null || connectionsList.Count == 0) 
				{
					String errorMessage = "Error in " + spConnections;
					context.Response.Redirect(Constants.WEB_PREFIX + Constants.ERROR_PAGE + errorMessage);
				}
				return connectionsList;
			}
			else
			{
				return null;
			}
	}

		public bool IsReusable {
			get { return true; }
		}
	}
}